@@ -12,6 +12,8 @@ from robotoff.types import (
     NeuralCategoryClassifierModel,
     ObjectDetectionModel,
     PredictionType,
+    ProductIdentifier,
+    ServerType,
     WorkerQueue,
 )
 
@@ -46,11 +48,13 @@ def run_worker(
 
 @app.command()
 def regenerate_ocr_insights(
-    barcode: str = typer.Argument(..., help="Barcode of the product")
+    barcode: str = typer.Argument(..., help="Barcode of the product"),
+    server_type: ServerType = typer.Option(
+        ServerType.off, help="Server type of the product"
+    ),
 ) -> None:
     """Regenerate OCR predictions/insights for a specific product and import
     them."""
-    from robotoff import settings
     from robotoff.insights import importer
     from robotoff.insights.extraction import (
         DEFAULT_OCR_PREDICTION_TYPES,
@@ -63,7 +67,8 @@ def regenerate_ocr_insights(
 
     logger = get_logger()
 
-    product = get_product(barcode, ["images"])
+    product_id = ProductIdentifier(barcode, server_type)
+    product = get_product(product_id, ["images"])
     if product is None:
         raise ValueError(f"product not found: {barcode}")
 
@@ -72,15 +77,13 @@ def regenerate_ocr_insights(
         if not image_id.isdigit():
             continue
 
-        ocr_url = generate_json_ocr_url(barcode, image_id)
+        ocr_url = generate_json_ocr_url(product_id, image_id)
         predictions += extract_ocr_predictions(
-            barcode, ocr_url, DEFAULT_OCR_PREDICTION_TYPES
+            product_id, ocr_url, DEFAULT_OCR_PREDICTION_TYPES
         )
 
     with db:
-        import_result = importer.import_insights(
-            predictions, settings.BaseURLProvider.server_domain()
-        )
+        import_result = importer.import_insights(predictions, server_type)
         logger.info(import_result)
 
 
@@ -92,6 +95,9 @@ def generate_ocr_predictions(
     prediction_type: PredictionType = typer.Argument(
         ..., help="Type of the predictions to generate (label, brand,...)"
     ),
+    server_type: ServerType = typer.Option(
+        ServerType.off, help="Server type of the archive"
+    ),
     output: Optional[Path] = typer.Option(
         None,
         help="File to write output to, stdout if not specified. Gzipped output are supported.",
@@ -104,7 +110,7 @@ def generate_ocr_predictions(
     from robotoff.utils import get_logger
 
     get_logger()
-    insights.run_from_ocr_archive(input_path, prediction_type, output)
+    insights.run_from_ocr_archive(input_path, prediction_type, server_type, output)
 
 
 @app.command()
@@ -137,6 +143,9 @@ def download_dataset(minify: bool = False) -> None:
 @app.command()
 def categorize(
     barcode: str,
+    server_type: ServerType = typer.Option(
+        ServerType.off, help="Server type of the product"
+    ),
     deepest_only: bool = False,
     model_name: NeuralCategoryClassifierModel = typer.Option(
         NeuralCategoryClassifierModel.keras_image_embeddings_3_0,
@@ -159,20 +168,23 @@ def categorize(
 
     get_logger(level=logging.DEBUG)
 
-    product = get_product(barcode)
+    product_id = ProductIdentifier(barcode, server_type)
+    product = get_product(product_id)
     if product is None:
-        print(f"Product {barcode} not found")
+        print(f"{product_id} not found")
         return
 
     predictions, _ = CategoryClassifier(
         get_taxonomy(TaxonomyType.category.name, offline=True)
-    ).predict(product, deepest_only, threshold=threshold, model_name=model_name)
+    ).predict(
+        product, product_id, deepest_only, threshold=threshold, model_name=model_name
+    )
 
     if predictions:
         for prediction in predictions:
             print(f"{prediction.value_tag}: {prediction.confidence}")
     else:
-        print(f"Nothing predicted for product {barcode}")
+        print(f"Nothing predicted for {product_id}")
 
 
 @app.command()
@@ -191,6 +203,9 @@ def import_insights(
     generate_from: Optional[pathlib.Path] = typer.Option(
         None, help="Input path of the OCR archive, is incompatible with --input-path"
     ),
+    server_type: ServerType = typer.Option(
+        ServerType.off, help="Server type of the product"
+    ),
 ) -> None:
     """Import insights from a prediction JSONL archive (with --input-path
     option), or generate them on the fly from an OCR archive (with
@@ -198,7 +213,6 @@ def import_insights(
     import tqdm
     from more_itertools import chunked
 
-    from robotoff import settings
     from robotoff.cli.insights import generate_from_ocr_archive, insights_iter
     from robotoff.insights import importer
     from robotoff.models import db
@@ -211,7 +225,9 @@ def import_insights(
         if prediction_type is None:
             sys.exit("Required option: --prediction-type")
 
-        predictions = generate_from_ocr_archive(generate_from, prediction_type)
+        predictions = generate_from_ocr_archive(
+            generate_from, prediction_type, server_type
+        )
     elif input_path is not None:
         logger.info(f"Importing insights from {input_path}")
         predictions = insights_iter(input_path)
@@ -224,9 +240,7 @@ def import_insights(
         ):
             # Create a new transaction for every batch
             with db.atomic():
-                import_results = importer.import_insights(
-                    prediction_batch, settings.BaseURLProvider.server_domain()
-                )
+                import_results = importer.import_insights(prediction_batch, server_type)
                 logger.info(import_results)
 
 
@@ -236,6 +250,9 @@ def refresh_insights(
         None,
         help="Refresh a specific product. If not provided, all products are updated",
     ),
+    server_type: ServerType = typer.Option(
+        ServerType.off, help="Server type of the product"
+    ),
     batch_size: int = typer.Option(
         100, help="Number of products to send in a worker tasks"
     ),
@@ -249,7 +266,6 @@ def refresh_insights(
     from more_itertools import chunked
     from peewee import fn
 
-    from robotoff import settings
     from robotoff.insights.importer import refresh_insights as refresh_insights_
     from robotoff.models import Prediction as PredictionModel
     from robotoff.models import db
@@ -260,23 +276,24 @@ def refresh_insights(
     logger = get_logger()
 
     if barcode is not None:
-        logger.info(f"Refreshing product {barcode}")
+        product_id = ProductIdentifier(barcode, server_type)
+        logger.info(f"Refreshing {product_id}")
         with db:
-            imported = refresh_insights_(
-                barcode, settings.BaseURLProvider.server_domain()
-            )
+            imported = refresh_insights_(product_id)
         logger.info(f"Refreshed insights: {imported}")
     else:
         logger.info("Launching insight refresh on full database")
         with db:
-            barcodes = [
-                barcode
-                for (barcode,) in PredictionModel.select(
+            product_ids = [
+                ProductIdentifier(barcode, server_type)
+                for (barcode, server_type) in PredictionModel.select(
                     fn.Distinct(PredictionModel.barcode)
-                ).tuples()
+                )
+                .where(PredictionModel.server_type == server_type.name)
+                .tuples()
             ]
 
-        batches = list(chunked(barcodes, batch_size))
+        batches = list(chunked(product_ids, batch_size))
         confirm = typer.confirm(
             f"{len(batches)} jobs are going to be launched, confirm?"
         )
@@ -285,18 +302,20 @@ def refresh_insights(
             return
 
         logger.info("Adding refresh_insights jobs in queue...")
-        for barcode_batch in tqdm.tqdm(batches, desc="barcode batch"):
+        for product_id_batch in tqdm.tqdm(batches, desc="barcode batch"):
             enqueue_job(
                 refresh_insights_job,
                 low_queue,
                 job_kwargs={"result_ttl": 0, "timeout": "5m"},
-                barcodes=barcode_batch,
-                server_domain=settings.BaseURLProvider.server_domain(),
+                product_ids=product_id_batch,
             )
 
 
 @app.command()
 def import_images_in_db(
+    server_type: ServerType = typer.Option(
+        ServerType.off, help="Server type of the product"
+    ),
     batch_size: int = typer.Option(
         500, help="Number of items to send in a worker tasks"
     ),
@@ -306,10 +325,9 @@ def import_images_in_db(
     import tqdm
     from more_itertools import chunked
 
-    from robotoff import settings
     from robotoff.models import ImageModel, db
     from robotoff.off import generate_image_path
-    from robotoff.products import get_product_store
+    from robotoff.products import DBProductStore, get_product_store
     from robotoff.utils import get_logger
     from robotoff.workers.queues import enqueue_job, low_queue
     from robotoff.workers.tasks.import_image import save_image_job
@@ -319,18 +337,25 @@ def import_images_in_db(
     with db:
         logger.info("Fetching existing images in DB...")
         existing_images = set(
-            ImageModel.select(ImageModel.barcode, ImageModel.image_id).tuples()
+            ImageModel.select(ImageModel.barcode, ImageModel.image_id)
+            .where(ImageModel.server_type == server_type.name)
+            .tuples()
         )
 
-    store = get_product_store()
-    to_add = []
+    store: DBProductStore = get_product_store(server_type)
+    to_add: list[tuple[ProductIdentifier, str]] = []
     for product in tqdm.tqdm(
         store.iter_product(projection=["images", "code"]), desc="product"
     ):
         barcode = product.barcode
         for image_id in (id_ for id_ in product.images.keys() if id_.isdigit()):
             if (barcode, image_id) not in existing_images:
-                to_add.append((barcode, generate_image_path(barcode, image_id)))
+                to_add.append(
+                    (
+                        ProductIdentifier(barcode, server_type),
+                        generate_image_path(barcode, image_id),
+                    )
+                )
 
     batches = list(chunked(to_add, batch_size))
     if typer.confirm(
@@ -342,12 +367,15 @@ def import_images_in_db(
                 low_queue,
                 job_kwargs={"result_ttl": 0},
                 batch=batch,
-                server_domain=settings.BaseURLProvider.server_domain(),
+                server_type=server_type,
             )
 
 
 @app.command()
 def run_object_detection_model(
+    server_type: ServerType = typer.Option(
+        ServerType.off, help="Server type of the product"
+    ),
     model_name: ObjectDetectionModel = typer.Argument(
         ..., help="Name of the object detection model"
     ),
@@ -370,7 +398,6 @@ def run_object_detection_model(
     import tqdm
     from peewee import JOIN
 
-    from robotoff import settings
     from robotoff.models import ImageModel, ImagePrediction, db
     from robotoff.off import generate_image_url
     from robotoff.utils import text_file_iter
@@ -412,15 +439,17 @@ def run_object_detection_model(
                     ),
                 )
                 .where(
-                    ImagePrediction.model_name.is_null()
-                    & (ImageModel.deleted == False)  # noqa: E712
+                    ImageModel.server_type
+                    == server_type.name
+                    & ImagePrediction.model_name.is_null()
+                    & (ImageModel.deleted == False),  # noqa: E712
                 )
                 .tuples()
             )
             if limit:
                 query = query.limit(limit)
             image_urls = [
-                generate_image_url(barcode, image_id)
+                generate_image_url(ProductIdentifier(barcode, server_type), image_id)
                 for barcode, image_id in query
                 if barcode.isdigit()
             ]
@@ -428,13 +457,14 @@ def run_object_detection_model(
     if typer.confirm(f"{len(image_urls)} jobs are going to be launched, confirm?"):
         for image_url in tqdm.tqdm(image_urls, desc="image"):
             barcode = get_barcode_from_url(image_url)
+            if barcode is None:
+                raise RuntimeError()
             enqueue_job(
                 func,
                 low_queue,
                 job_kwargs={"result_ttl": 0},
-                barcode=barcode,
+                product_id=ProductIdentifier(barcode, server_type),
                 image_url=image_url,
-                server_domain=settings.BaseURLProvider.server_domain(),
             )
 
 
@@ -455,9 +485,12 @@ def init_elasticsearch(load_data: bool = True) -> None:
 
 @app.command()
 def add_logo_to_ann(
+    server_type: ServerType = typer.Option(
+        ServerType.off, help="Server type of the logos"
+    ),
     sleep_time: float = typer.Option(
         0.0, help="Time to sleep between each query (in s)"
-    )
+    ),
 ) -> None:
     """Index all missing logos in Elasticsearch ANN index."""
     import logging
@@ -492,7 +525,7 @@ def add_logo_to_ann(
         )
         for logo_embedding_batch in chunked(logo_embedding_iter, 500):
             try:
-                add_logos_to_ann(es_client, logo_embedding_batch)
+                add_logos_to_ann(es_client, logo_embedding_batch, server_type)
                 added += len(logo_embedding_batch)
             except BulkIndexError as e:
                 logger.info("Request error during logo addition to ANN", exc_info=e)
@@ -507,6 +540,7 @@ def add_logo_to_ann(
 def refresh_logo_nearest_neighbors(
     day_offset: int = typer.Option(7, help="Number of days since last refresh", min=1),
     batch_size: int = typer.Option(500, help="Number of logos to process at once"),
+    server_type: ServerType = typer.Option(ServerType.off, help="Server type"),
 ):
     """Refresh each logo nearest neighbors if the last refresh is more than
     `day_offset` days old."""
@@ -522,7 +556,7 @@ def refresh_logo_nearest_neighbors(
     logger.info("Starting refresh of logo nearest neighbors")
 
     with db.connection_context():
-        refresh_nearest_neighbors(day_offset, batch_size)
+        refresh_nearest_neighbors(server_type, day_offset, batch_size)
 
 
 @app.command()
@@ -593,6 +627,7 @@ def import_logos(
     batch_size: int = typer.Option(
         1024, help="Number of predictions to insert in DB in a single SQL transaction"
     ),
+    server_type: ServerType = typer.Option(ServerType.off, help="Server type"),
 ) -> None:
     """Import object detection predictions for universal-logo-detector model.
 
@@ -626,6 +661,7 @@ def import_logos(
                 ObjectDetectionModel.universal_logo_detector
             ],
             batch_size,
+            server_type,
         )
 
     logger.info("%s image predictions created", imported)
@@ -661,5 +697,51 @@ def export_logos(
         dump_jsonl(output, query.dicts().iterator())
 
 
+@app.command()
+def import_image_webhook(
+    image_url: str = typer.Argument(
+        ...,
+        help="URL of the image to import to the output file, can either have .jsonl or .jsonl.gz as "
+        "extension",
+    ),
+    server_domain: str = typer.Option(
+        "api.openfoodfacts.net", help="Server domain to use for image import"
+    ),
+) -> None:
+    """Import an image in Robotoff by calling POST /api/v1/images/import.
+
+    The OCR URL will be generated automatically from the image URL. This is a
+    helper CLI command created for debugging/local developpement only.
+    """
+    import os
+
+    from robotoff.off import get_barcode_from_url
+    from robotoff.utils import get_logger, http_session
+
+    logger = get_logger()
+    ocr_url = image_url.replace(".jpg", ".json")
+    barcode = get_barcode_from_url(image_url)
+
+    # Use `api` alias instead of localhost if we're running in a docker container
+    domain = (
+        "http://localhost:5500"
+        if bool(os.environ.get("IN_DOCKER_CONTAINER", False))
+        else "http://api:5500"
+    )
+    r = http_session.post(
+        f"{domain}/api/v1/images/import",
+        data={
+            "barcode": barcode,
+            "image_url": image_url,
+            "ocr_url": ocr_url,
+            "server_domain": server_domain,
+        },
+    )
+    if not r.ok:
+        logger.info("HTTP error (%s) during image import: %s", r.status_code, r.text)
+    else:
+        logger.info("Robotoff response: %s", r.json())
+
+
 def main() -> None:
     app()
