@@ -1,4 +1,4 @@
-from robotoff.utils.text import strip_accents_ascii
+from robotoff.utils.text import strip_accents_v1
 
 
 def normalize_emb_code(emb_code: str):
@@ -10,7 +10,7 @@ def normalize_emb_code(emb_code: str):
         emb_code.strip().lower().replace(" ", "").replace("-", "").replace(".", "")
     )
 
-    emb_code = strip_accents_ascii(emb_code)
+    emb_code = strip_accents_v1(emb_code)
 
     """if the code ends with "ce" replace it with "ec"
     here "fr40261001ce" becomes "fr40261001ec"
