@@ -5,21 +5,29 @@ import functools
 import gzip
 import json
 import os
-import pathlib
 import shutil
 import tempfile
+from pathlib import Path
 from typing import Iterable, Iterator, Optional, Union
 
 import requests
 from pymongo import MongoClient
 
 from robotoff import settings
-from robotoff.mongo import MONGO_CLIENT_CACHE
-from robotoff.types import JSONType
+from robotoff.types import JSONType, ProductIdentifier, ServerType
 from robotoff.utils import get_logger, gzip_jsonl_iter, http_session, jsonl_iter
 
 logger = get_logger(__name__)
 
+MONGO_SELECTION_TIMEOUT_MS = 10_0000
+
+
+@functools.cache
+def get_mongo_client() -> Optional[MongoClient]:
+    return MongoClient(
+        settings.MONGO_URI, serverSelectionTimeoutMS=MONGO_SELECTION_TIMEOUT_MS
+    )
+
 
 def get_image_id(image_path: str) -> Optional[str]:
     """Return the image ID from an image path.
@@ -28,7 +36,7 @@ def get_image_id(image_path: str) -> Optional[str]:
     :return: the image ID ("2" in the previous example) or None if the image
     is not "raw" (not digit-numbered)
     """
-    image_id = pathlib.Path(image_path).stem
+    image_id = Path(image_path).stem
 
     if image_id.isdigit():
         return image_id
@@ -37,7 +45,7 @@ def get_image_id(image_path: str) -> Optional[str]:
 
 
 def is_valid_image(images: JSONType, image_path: str) -> bool:
-    image_id = pathlib.Path(image_path).stem
+    image_id = Path(image_path).stem
     if not image_id.isdigit():
         return False
 
@@ -69,7 +77,7 @@ def is_special_image(
     if not is_valid_image(images, image_path):
         return False
 
-    image_id = pathlib.Path(image_path).stem
+    image_id = Path(image_path).stem
 
     for image_key, image_data in images.items():
         if (
@@ -85,7 +93,7 @@ def is_special_image(
     return False
 
 
-def minify_product_dataset(dataset_path: pathlib.Path, output_path: pathlib.Path):
+def minify_product_dataset(dataset_path: Path, output_path: Path):
     if dataset_path.suffix == ".gz":
         jsonl_iter_func = gzip_jsonl_iter
     else:
@@ -120,7 +128,7 @@ def save_product_dataset_etag(etag: str):
 
 def fetch_dataset(minify: bool = True) -> bool:
     with tempfile.TemporaryDirectory() as tmp_dir:
-        output_dir = pathlib.Path(tmp_dir)
+        output_dir = Path(tmp_dir)
         output_path = output_dir / "products.jsonl.gz"
         etag = download_dataset(output_path)
 
@@ -172,7 +180,7 @@ def download_dataset(output_path: os.PathLike) -> str:
     return current_etag
 
 
-def is_valid_dataset(dataset_path: pathlib.Path) -> bool:
+def is_valid_dataset(dataset_path: Path) -> bool:
     """Check the dataset integrity: readable end to end and with a minimum number of products.
     This is used to spot corrupted archive files."""
     dataset = ProductDataset(dataset_path)
@@ -339,9 +347,14 @@ class ProductStream:
     def iter(self) -> Iterable[JSONType]:
         return iter(self)
 
-    def iter_product(self) -> Iterable["Product"]:
+    def iter_product(
+        self, projection: Optional[list[str]] = None
+    ) -> Iterable["Product"]:
         for item in self:
-            yield Product(item)
+            projected_item = (
+                {k: item[k] for k in projection if k in item} if projection else item
+            )
+            yield Product(projected_item)
 
     def collect(self) -> list[JSONType]:
         return list(self)
@@ -356,12 +369,10 @@ class ProductDataset:
         self.jsonl_path = jsonl_path
 
     def stream(self) -> ProductStream:
-        json_path_str = str(self.jsonl_path)
-
-        if json_path_str.endswith(".gz"):
-            iterator = gzip_jsonl_iter(json_path_str)
+        if str(self.jsonl_path).endswith(".gz"):
+            iterator = gzip_jsonl_iter(self.jsonl_path)
         else:
-            iterator = jsonl_iter(json_path_str)
+            iterator = jsonl_iter(self.jsonl_path)
 
         return ProductStream(iterator)
 
@@ -444,22 +455,26 @@ class MemoryProductStore(ProductStore):
         return len(self.store)
 
     @classmethod
-    def load_from_path(cls, path: str):
+    def load_from_path(cls, path: Path, projection: Optional[list[str]] = None):
         logger.info("Loading product store")
+
+        if projection is not None and "code" not in projection:
+            raise ValueError("at least `code` must be in projection")
+
         ds = ProductDataset(path)
         stream = ds.stream()
 
         store: dict[str, Product] = {}
 
-        for product in stream.iter_product():
+        for product in stream.iter_product(projection):
             if product.barcode:
                 store[product.barcode] = product
 
         return cls(store)
 
     @classmethod
-    def load_min(cls):
-        return cls.load_from_path(settings.JSONL_MIN_DATASET_PATH)
+    def load_min(cls, projection: Optional[list[str]] = None):
+        return cls.load_from_path(settings.JSONL_MIN_DATASET_PATH, projection)
 
     @classmethod
     def load_full(cls):
@@ -473,21 +488,23 @@ class MemoryProductStore(ProductStore):
 
 
 class DBProductStore(ProductStore):
-    def __init__(self, client: MongoClient):
+    def __init__(self, server_type: ServerType, client: MongoClient):
         self.client = client
-        self.db = self.client.off
+        self.server_type = server_type
+        db_name = server_type.value
+        self.db = self.client[db_name]
         self.collection = self.db.products
 
     def __len__(self):
         return len(self.collection.estimated_document_count())
 
     def get_product(
-        self, barcode: str, projection: Optional[list[str]] = None
+        self, product_id: ProductIdentifier, projection: Optional[list[str]] = None
     ) -> Optional[JSONType]:
-        return self.collection.find_one({"code": barcode}, projection)
+        return self.collection.find_one({"code": product_id.barcode}, projection)
 
-    def __getitem__(self, barcode: str) -> Optional[Product]:
-        product = self.get_product(barcode)
+    def __getitem__(self, product_id: ProductIdentifier) -> Optional[Product]:
+        product = self.get_product(product_id)
 
         if product:
             return Product(product)
@@ -495,34 +512,33 @@ class DBProductStore(ProductStore):
         return None
 
     def __iter__(self):
-        yield from self.iter()
+        if self.collection is not None:
+            yield from self.iter()
 
     def iter_product(self, projection: Optional[list[str]] = None):
-        yield from (Product(p) for p in self.collection.find(projection=projection))
+        if self.collection is not None:
+            yield from (Product(p) for p in self.collection.find(projection=projection))
 
 
-@functools.cache
-def get_min_product_store() -> ProductStore:
+def get_min_product_store(projection: Optional[list[str]] = None) -> ProductStore:
     logger.info("Loading product store in memory...")
-    ps = MemoryProductStore.load_min()
+    ps = MemoryProductStore.load_min(projection)
     logger.info("product store loaded (%s items)", len(ps))
     return ps
 
 
-def get_product_store() -> DBProductStore:
-    mongo_client = MONGO_CLIENT_CACHE.get()
-    return DBProductStore(client=mongo_client)
+def get_product_store(server_type: ServerType) -> DBProductStore:
+    return DBProductStore(server_type, client=get_mongo_client())
 
 
 def get_product(
-    barcode: str, projection: Optional[list[str]] = None
+    product_id: ProductIdentifier, projection: Optional[list[str]] = None
 ) -> Optional[JSONType]:
     """Get product from MongoDB.
 
-    :param barcode: barcode of the product to fetch
+    :param product_id: identifier of the product to fetch
     :param projection: list of fields to retrieve, if not provided all fields
     are queried
     :return: the product as a dict or None if it was not found
     """
-    mongo_client = MONGO_CLIENT_CACHE.get()
-    return mongo_client.off.products.find_one({"code": barcode}, projection)
+    return get_product_store(product_id.server_type).get_product(product_id, projection)
