@@ -1,5 +1,4 @@
 import datetime
-import functools
 import os
 import uuid
 from typing import Iterable
@@ -9,6 +8,7 @@ from apscheduler.events import EVENT_JOB_ERROR
 from apscheduler.executors.pool import ThreadPoolExecutor
 from apscheduler.jobstores.memory import MemoryJobStore
 from apscheduler.schedulers.blocking import BlockingScheduler
+from playhouse.postgres_ext import ServerSide
 from sentry_sdk import capture_exception
 
 from robotoff import settings, slack
@@ -21,7 +21,7 @@ from robotoff.metrics import (
     save_facet_metrics,
     save_insight_metrics,
 )
-from robotoff.models import ProductInsight, db, with_db
+from robotoff.models import Prediction, ProductInsight, db, with_db
 from robotoff.prediction.category.matcher import predict_from_dataset
 from robotoff.products import (
     Product,
@@ -30,6 +30,7 @@ from robotoff.products import (
     get_min_product_store,
     has_dataset_changed,
 )
+from robotoff.types import ProductIdentifier, ServerType
 from robotoff.utils import get_logger
 
 from .latent import generate_quality_facets
@@ -40,9 +41,10 @@ logger = get_logger(__name__)
 
 
 # Note: we do not use with_db, for atomicity is handled in annotator
-def process_insights():
+def process_insights() -> None:
     with db.connection_context():
         processed = 0
+        insight: ProductInsight
         for insight in (
             ProductInsight.select()
             .where(
@@ -54,7 +56,7 @@ def process_insights():
         ):
             try:
                 logger.info(
-                    "Annotating insight %s (product: %s)", insight.id, insight.barcode
+                    "Annotating insight %s (%s)", insight.id, insight.get_product_id()
                 )
                 annotation_result = annotate(insight, 1, update=True)
                 processed += 1
@@ -69,18 +71,20 @@ def process_insights():
                 # continue to the next one
                 # Note: annotator already rolled-back the transaction
                 logger.exception(
-                    f"exception {e} while handling annotation of insight %s (product) %s",
+                    f"exception {e} while handling annotation of insight %s (%s)",
                     insight.id,
-                    insight.barcode,
+                    insight.get_product_id(),
                 )
     logger.info("%d insights processed", processed)
 
 
 @with_db
-def refresh_insights(with_deletion: bool = False):
-    deleted = 0
-    updated = 0
-    product_store = get_min_product_store()
+def refresh_insights(with_deletion: bool = True) -> None:
+    product_store = get_min_product_store(
+        ["code", "brands_tags", "countries_tags", "unique_scans_n"]
+    )
+    # Only OFF is currently supported
+    server_type = ServerType.off
 
     datetime_threshold = datetime.datetime.utcnow().replace(
         hour=0, minute=0, second=0, microsecond=0
@@ -95,21 +99,23 @@ def refresh_insights(with_deletion: bool = False):
         )
         return
 
-    for insight in (
-        ProductInsight.select()
-        .where(
+    insight: ProductInsight
+    deleted = 0
+    updated = 0
+    for insight in ServerSide(
+        ProductInsight.select().where(
             ProductInsight.annotation.is_null(),
             ProductInsight.timestamp <= datetime_threshold,
-            ProductInsight.server_domain == settings.BaseURLProvider.server_domain(),
+            ProductInsight.server_type == server_type.name,
         )
-        .iterator()
     ):
+        product_id = insight.get_product_id()
         product: Product = product_store[insight.barcode]
 
         if product is None:
             if with_deletion:
                 # Product has been deleted from OFF
-                logger.info("Product with barcode {} deleted".format(insight.barcode))
+                logger.info("%s deleted, deleting insight %s", product_id, insight)
                 deleted += 1
                 insight.delete_instance()
         else:
@@ -118,48 +124,71 @@ def refresh_insights(with_deletion: bool = False):
             if insight_updated:
                 updated += 1
 
-    logger.info("{} insights deleted".format(deleted))
-    logger.info("{} insights updated".format(updated))
+    prediction: Prediction
+    deleted = 0
+    for prediction in ServerSide(
+        Prediction.select().where(
+            Prediction.timestamp <= datetime_threshold,
+            Prediction.server_type == server_type.name,
+        )
+    ):
+        product = product_store[ProductIdentifier(prediction.barcode, server_type)]
+
+        if product is None:
+            if with_deletion:
+                # Product has been deleted from OFF
+                logger.info(
+                    "%s deleted, deleting prediction %s", product_id, prediction
+                )
+                deleted += 1
+                prediction.delete_instance()
+
+    logger.info("%s prediction deleted", deleted)
 
 
 def update_insight_attributes(product: Product, insight: ProductInsight) -> bool:
-    to_update = False
+    updated_fields = []
     if insight.brands != product.brands_tags:
-        logger.info(
-            "Updating brand {} -> {} ({})".format(
-                insight.brands, product.brands_tags, product.barcode
-            )
+        logger.debug(
+            "Updating brand %s -> %s (%s)",
+            insight.brands,
+            product.brands_tags,
+            insight.get_product_id(),
         )
-        to_update = True
+        updated_fields.append("brands")
         insight.brands = product.brands_tags
 
     if insight.countries != product.countries_tags:
-        logger.info(
-            "Updating countries {} -> {} ({})".format(
-                insight.countries, product.countries_tags, product.barcode
-            )
+        logger.debug(
+            "Updating countries %s -> %s (%s)",
+            insight.countries,
+            product.countries_tags,
+            insight.get_product_id(),
         )
-        to_update = True
+        updated_fields.append("countries")
         insight.countries = product.countries_tags
 
     if insight.unique_scans_n != product.unique_scans_n:
-        logger.info(
-            "Updating unique scan count {} -> {} ({})".format(
-                insight.unique_scans_n, product.unique_scans_n, product.barcode
-            )
+        logger.debug(
+            "Updating unique scan count %s -> %s (%s)",
+            insight.unique_scans_n,
+            product.unique_scans_n,
+            insight.get_product_id(),
         )
-        to_update = True
+        updated_fields.append("unique_scans_n")
         insight.unique_scans_n = product.unique_scans_n
 
-    if to_update:
-        insight.save()
+    if updated_fields:
+        # Only update selected field with bulk_update and a list of fields to update
+        ProductInsight.bulk_update([insight], fields=updated_fields)
 
-    return to_update
+    return bool(updated_fields)
 
 
 @with_db
-def mark_insights():
+def mark_insights() -> int:
     marked = 0
+    insight: ProductInsight
     for insight in (
         ProductInsight.select()
         .where(
@@ -170,8 +199,9 @@ def mark_insights():
         .iterator()
     ):
         logger.info(
-            "Marking insight {} as processable automatically "
-            "(product: {})".format(insight.id, insight.barcode)
+            "Marking insight %s as processable automatically (%s)",
+            insight.id,
+            insight.get_product_id(),
         )
         insight.process_after = datetime.datetime.utcnow() + datetime.timedelta(
             minutes=10
@@ -204,7 +234,7 @@ def _update_data():
         logger.exception("Exception during ES indices creation", exc_info=e)
 
 
-def generate_insights():
+def generate_insights() -> None:
     """Generate and import category insights from the latest dataset dump, for
     products added at day-1."""
     logger.info("Generating new category insights")
@@ -218,7 +248,8 @@ def generate_insights():
     with db:
         import_result = import_insights(
             product_predictions_iter,
-            server_domain=settings.BaseURLProvider.server_domain(),
+            # Currently the JSONL dataset is OFF-only
+            server_type=ServerType.off,
         )
     logger.info(import_result)
 
@@ -273,7 +304,7 @@ def run():
     #   are no longer applicable.
     # - Updating insight attributes.
     scheduler.add_job(
-        functools.partial(refresh_insights, with_deletion=True),
+        refresh_insights,
         "cron",
         day="*",
         hour="4",
